import pandas as pd
import openpyxl
import random
import numpy as np
import math

def estimated(N, p, m, x, w, b):
    s = np.zeros((N, m))
    y_est = np.zeros(N)
    for k in range(N):
        for i in range(m):
            for j in range(p):
                s[k, i] += x[k, j] * w[j, i]

    for k in range(N):
        s1 = 0
        for i in range(m):
            s1 += b[i] / (1 + math.exp(-s[k, i]))
        y_est[k] = s1
    return y_est

def error(N, y, y_est):
    E = 0
    for k in range(N):
        E += pow(y[k] - y_est[k], 2)
    return E

def derivatives(N, p, m, x, y, y_est, w, b):
    s = np.zeros((N, m))
    der_w = np.zeros((p, m))
    der_b = np.zeros(m)
    for k in range(N):
        for i in range(m):
            for j in range(p):
                s[k, i] += x[k, j] * w[j, i]
    for i in range(m):
        for j in range(p):
            sum1 = 0
            for k in range(N):
                r1 = y[k] - y_est[k]
                r2 = math.exp(-s[k, i]) / pow(1 + math.exp(-s[k, i]), 2)
                sum1 += r1 * r2 * x[k, j] * b[j]
            der_w[j, i] = (-2) * sum1
            sum2 = 0
            for k in range(N):
                sum2 += (y[k] - y_est[k]) / (1 + math.exp(-s[k, i]))
            der_b[i] = (-2) * sum2
    return der_w, der_b

df = pd.read_excel('x_data.xlsx')
x = df.to_numpy()
df = pd.read_excel('y_data.xlsx')
y = df.to_numpy()

N = len(x)
p = len(x[0])
m = 10
ni = 0.0001

w = np.zeros((p, m))
for j in range(p):
    for i in range(m):
        w[j, i] = -0.1 + random.random() * 0.2

b = np.zeros(m)
for i in range(m):
    b[i] = -0.01 + random.random() * 0.02

y_est = estimated(N, p, m, x, w, b)
E = error(N, y, y_est)
RMSE = math.sqrt(E / N)
print (RMSE)

for i in range(10):
  der_w, der_b = derivatives(N, p, m, x, y, y_est, w, b)
  for j in range(p):
      for i in range(m):
          w[j, i] -= - ni * der_w[j, i]
  for i in range(m):
      b[i] -= ni * der_b[i]
  
  y_est = estimated(N, p, m, x, w, b)
  E = error(N, y, y_est)
  RMSE = math.sqrt(E / N)
  print (RMSE)

print (w)
print (b)
  
